/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
*    initial Motorola version by Bowling & Grippo (2002)  *
*            http://www.bgsoflex.com/megasquirt.html      *
\*********************************************************/


#ifndef COMM_H
#define COMM_H

#include <inttypes.h>
#include <avr/io.h>
/* must be power of 2 */
#define CMD_QUEUE_SIZE 8

#ifdef TRIGGERLOG_PROTOCOL
/* HDLC special characters */
#define HDLC_END         0x7E            /* indicates end of frame; also sent at the beginning to flush any trash that accumulated in the receiver */
#define HDLC_ESC         0x7D            /* indicates byte stuffing      */
#define HDLC_ESCAPE(x)   ((x)^0x20)
#define HDLC_UNESCAPE(x) ((x)^0x20)

#define RT_STUFFED_TYPE 32 /*byte stuffed realtime packet type id*/

#endif
/*
 * a structure to hold the private characters so they can also
 * be used as a private word
 */
struct privch_t {
	uint8_t a;
	uint8_t b;
};

union cmd_queue_p1 {
	uint8_t * private;
	uint8_t (*privatefunc)(uint8_t i);
};

union cmd_queue_p2 {
	struct privch_t privch;
	uint16_t privw;
};

struct cmd_queue_t {
	uint8_t (*func)(uint8_t i);
	uint8_t len;		/* number of bytes to send */
	uint8_t count;		/* number of bytes already sent */
	union cmd_queue_p1 p1;
	union cmd_queue_p2 p2;
	uint8_t seq;		/* sequence number used in sipr */
};

/*
 * We could enumerate instead, but ansi-C promotes enumerations to
 * integers (16 bit), no need for that!
 */
#define CMD    0
#define OFFSET 1
#define VALUE  2
#define AIM_DATA  3
#define AIM_CANDIDATE 4

#ifdef TRIGGERLOG_PROTOCOL
extern uint16_t commcrc;/* crc for triggerlog*/
#endif
extern uint8_t comm_state;
extern uint8_t comm_prevrx; /* previous received byte */
extern uint8_t rx_offset;
extern uint8_t rx_cmd;
extern uint8_t cmd_queue_head, cmd_queue_tail;
extern struct cmd_queue_t cmd_queue[CMD_QUEUE_SIZE];

#define comm_buffer_full() (cmd_queue_head == ((cmd_queue_tail + 1) % CMD_QUEUE_SIZE))

#define comm_TX_plain_SIZE (13)
extern uint8_t comm_TX_plainbytes[comm_TX_plain_SIZE];

#define MT_RTVAR_COUNT (38)

/* prototypes */
void comm(uint8_t rx_data);
uint8_t sendConfigVar(uint8_t i);
void storeConfigPar(uint8_t offset, uint8_t value);
uint8_t sendRTVar(uint8_t i);
uint8_t MTcodeVer(uint8_t i);

uint8_t pushfunc(uint8_t (*f)(uint8_t i), uint8_t len);
uint8_t pushfuncpar(uint8_t (*f)(uint8_t i), uint8_t len, const uint8_t *);
uint8_t pushfuncpar2(uint8_t (*f)(uint8_t i), uint8_t len, const uint8_t * par, uint16_t p2);
uint8_t srl_byte(uint8_t);
uint8_t srl_generic(uint8_t);
uint8_t srl_generic_P(uint8_t addr);
uint8_t srl_generic_P_upper(uint8_t addr);
uint8_t srl_direct(uint8_t);
#ifdef TRIGGERLOG_PROTOCOL
uint8_t MTsendRTvar_triggerlog(uint8_t i);
uint8_t TriggerlogAnytrig( uint8_t i );
uint8_t SendPage_stuffed( uint8_t i );
#endif
void enable_transmit_interrupt(void);
void initUART(void);

/* user defined baudrate */
static __inline__ void change_baudrate( uint8_t newbaud ){
	UBRR0H = 0;
	UBRR0L = newbaud;
}

enum MegatunePageIdx
{
    MT_PAGE_IDX_CONFIG = 0,     /* config*/
    MT_PAGE_IDX_LAST = MT_PAGE_IDX_CONFIG/*last page idx*/
};

/* sendpage error codes*/
#define SENDPAGE_ERROR_NO_PAGEONID 1
/* extern uint8_t new_baudrate; */

#endif
